// File : pylibgrass5.i
// SWIG interface for libgrass5
// Not all functions are wrapped, it should be enough to run grassinfo.py.
//
// probably only useful to generate Python bindings.
//	swig -python -shadow shapelib.i
//
//	tested with swig1.3a5
//
//	Copyright ImageLinks
//	Author: Bernhard Reiter <bernhard@intevation.de>
//
//	This file is part of libgrass5 and free software under LGPL.
//
// $Id$

%module pylibgrass5
%{
#include "gis.h"
%}

%typemap(python,in) string_allows_none {
	if($source==Py_None) {
		$target=NULL;
	} else {
		$target=PyString_AsString($source);
		if(!$target) {
			return NULL;
		}
	}
}

%inline %{
typedef char * string_allows_none;
%}

%typemap(python,out) char ** {
	int len=0,i;
	PyObject * stringobject;

	if(!$source) {
		$target=PyList_New(0);
	} else {

		/* count number of elements first */
		for(len=0;$source[len];len++);

		$target=PyList_New(len);
		if(!$target){
			 G_free_list($source);
			 return NULL;
		}
			
		for(i=0;i<len;i++) {
			stringobject=PyString_FromString($source[i]);
			if(!stringobject) return NULL;
			PyList_SetItem($target,i,stringobject);
			/* no DECREF needed, because 
			/*    PyList_SetItems steals reference*/
		}
		G_free_list($source);
	}
}

// creating a char** type where we only need the output
// the returned string will be immedeatly freed by G_free()
// as suggested in documentation for I_init_group()
//
%typemap(python,ignore) return_string (char * temp) {
	temp=NULL;
	$target=&temp;
}

%include typemaps.i

%typemap(python,argout) return_string (char * temp) {
	if($source && *$source) {
		$target=t_output_helper($target,PyString_FromString(*$source));
		G_free(*$source);
	} else {
		$target=t_output_helper($target,Py_None);
		Py_INCREF(Py_None);
	}
	
}

%inline %{
typedef char ** return_string;
%}


/*****************************************************************************
 *	from gis.h
 */

typedef int RASTER_MAP_TYPE;
#define CELL_TYPE 0
#define FCELL_TYPE 1
#define DCELL_TYPE 2

struct Cell_head
{
    Cell_head();
    ~Cell_head();
    int format       ;  /* max numer of bytes per cell minus 1          */
    int compressed   ;  /* 0 = uncompressed, 1 = compressed, -1 pre 3.0 */
    int rows, cols   ;  /* number of rows and columns in the data       */
    int proj         ;  /* Projection (see #defines above)              */
    int zone         ;  /* Projection zone                              */
    double ew_res    ;  /* East to West cell size                       */
    double ns_res    ;  /* North to South cell size                     */
    double north     ;  /* coordinates of layer                         */
    double south     ;
    double east      ;
    double west      ;
} ;

/*****************************************************************************
 *	from libgrass.h
 */

int G_gisinit_2( char *, string_allows_none a=NULL, 
			 string_allows_none b=NULL, 
			 string_allows_none c=NULL );


char ** G_fetch_list_element( char *, string_allows_none mapset=NULL);
char ** G_fetch_list_locations( string_allows_none gisdbase=NULL );
char ** G_fetch_list_mapsets( string_allows_none location_path=NULL );

/* not wrapped, because mainly used internally
 *
char ** G_fetch_list_files( char * directory );
 */

/* We do not need to wrap  G_free_list and G_add_to_list
 * because we only use the python lists
 *
void    G_free_list( char ** );
char  **G_add_to_list( char **, char * );
 */

char *G_path_peel_element( char * );
int G_check_cell( char *name,  
		return_string ret_mapset, return_string ret_cell );

int G_open_cell_old_2( char * );

int I_check_group( char *name, 
		return_string ret_mapset, 
		return_string ret_group );

char *G_get_cell_as_proj4( char *cell, char *mapset );
char *G_kv_to_proj4( struct Key_Value *in_proj_keys, 
                     struct Key_Value *in_units_keys );
int G_proj4_to_kv( char *proj4, 
                   struct Key_Value **out_proj_keys, 
                   struct Key_Value **out_units_keys );

int G__make_location( char *location_name, struct Cell_head *wind, 
    struct Key_Value *proj_info, struct Key_Value *proj_units,
    FILE *report_file );
void G_set_cellhd_from_projinfo( struct Cell_head *wind, 
                                 struct Key_Value *proj_info );

/* comp_proj.c */
int G_compare_projections( struct Key_Value *proj_info1, 
                       struct Key_Value *proj_units1, 
                       struct Key_Value *proj_info2, 
                       struct Key_Value *proj_units2 );

/*****************************************************************************
 *	from imagery.h
 *****************************************************************************
 */

%inline %{
struct Ref_Files
{
    char name[30];
    char mapset[30];
};

struct Ref_Color
{
    unsigned char *table      ;  /* color table for min-max values */
    unsigned char *index      ;  /* data translation index */
    unsigned char *buf        ;  /* data buffer for reading color file */
    int fd                    ;  /* for image i/o */
    CELL min, max             ;  /* min,max CELL values */
    int n                     ;  /* index into Ref_Files */
};

struct Ref
{
    int nfiles;
    struct Ref_Files *file;
    struct Ref_Color red;
    struct Ref_Color grn;
    struct Ref_Color blue;
};


struct Ref_Files * Ref_file_get_item(struct Ref_Files** file,int n) {
	return file[n];
}

%}

// Ref_Files * is actually the first element of an array,
// so we need this access function to get the other elements in python
%addmethods Ref_Files {
	struct Ref_Files * __getitem__(int i) {
		return (struct Ref_Files *) self+i;
	};
	
}



/* We can make it possible to delete Ref */
%addmethods Ref {
	Ref() {
		return(struct Ref *) malloc(sizeof(struct Ref));
	};
	~Ref(self) {
		if(self) {
		  	if(self->file)
				I_free_group_ref(self);
			free(self);
		}
	}
}



//%typemap(python,ignore) return_Ref (struct Ref *tmp) {
//	$target=tmp;
//}
//
//%typemap(python,argout) return_Ref {
//	$target=t_output_helper($target,$source);
//}
//
//%inline %{
//typedef struct Ref* return_Ref;
//%}

int I_get_group_ref(char *, struct Ref *);



/*****************************************************************************
 *	from gisdefs.h
 */

int G_get_cellhd(char *, char *, struct Cell_head *);

/* G_raster_map_type() might modify its first argument ...
   so we have to do something about it... */
%typemap(python,in) mutable_string {
	$target=G_strdup(PyString_AsString($source));
}

%typemap(python,argout) mutable_string {
	if (strcmp(PyString_AsString($arg),$source) == 0) {
		$target=t_output_helper($target,$arg);
		Py_INCREF($arg);
	}
	else {
		$target=t_output_helper($target,PyString_FromString($source));
	}
	G_free($source);
}

%inline %{
typedef char *mutable_string;
%}


RASTER_MAP_TYPE G_raster_map_type(mutable_string, char *);

int G_set_window(struct Cell_head *);
int G_close_cell(int);

